// UH module:
// Trigger pulse length >10 uSec
// Output pulse length 1us/0.58cm
// Max length 5m: 862uSec
// Timer clock 1MHz (sys clock/8) to measure pulse length in usec

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "BooleConst.h"
#include "timer_std_hlr.h"
#include "UH_dist_hlr.h"

const float uS_Pro_cm = 55.6;
int DistanceInt_cm;

//times in msec
#define MeasPeriod  1000
#define MeasFaliure  500

//--TIMER1 counts the pulse width of the input signal --
//use TIMER1
//          D7     D6     D5     D4     D3     D2     D1   D0
//TCCR1A: COM1A1 COM1A0 COM1B1 COM1B0    -		-    WGM11 WGM10
//           0      0      0      0      0     0      0     0
// Outputs disconnected

//          D7     D6     D5     D4     D3     D2     D1   D0
//TCCR1B: ICNC1   ICES1    -    WGM13  WGM12   CS12  CS11 CS10
//           0      1      0      0      0     0      1     0
//normal mode, internal clock/8, capture on rising edge

//TCCR1B: ICNC1   ICES1    -    WGM13  WGM12   CS12  CS11 CS1
//           0      0      0      0      0     0      1     0
//normal mode, internal clock/8, capture on falling edge



#define Timer1Enable  TCCR1B |= 0x02
#define Timer1Disable TCCR1B &= 0xf8
#define Timer1CaptureRisingEdge TCCR1B |= (1 << ICES1)
#define Timer1CaptureFalingEdge TCCR1B &= ~(1 << ICES1)

void TIMER1_Init(void)
{
	TCCR1A = 0x00; //Outputs disconnected, Normal mode
	Timer1Disable;
}


//---------- get number of counted pulses by capture unit -----------
static volatile char IC1count;
static volatile unsigned char NL1;
static volatile unsigned char NH1;
static volatile unsigned char NL2;
static volatile unsigned char NH2;

//static volatile unsigned char DistMeasReady;
volatile unsigned char DistMeasReady;

static volatile unsigned char TriggerReady;

#define InputCapture1IT_Enable TIMSK1 = TIMSK1 | (1<<ICIE1)
#define InputCapture1IT_Disable TIMSK1 = TIMSK1 & ~(1<<ICIE1)
#define InputCapture1IT_Clear TIMSK1 = TIMSK1 | (1<<ICF1)

ISR(TIMER1_CAPT_vect)
{
	
	IC1count++;					//capture events
	switch(IC1count)
	{
		case 1:	NL1 = ICR1L;	//read 1. capture value
		NH1 = ICR1H;
		Timer1CaptureFalingEdge;	// set capture sensing to falling edge
		InputCapture1IT_Clear;
		break;
		case 2:	NL2 = ICR1L;	//read 2. capture value
		NH2 = ICR1H;
		DistMeasReady = 1;
		InputCapture1IT_Disable;
		break;
	}
}


#define SetTriggerDir TriggerDirPort = TriggerDirPort | (1<< TriggerBit)
#define SET_TRIGGER TriggerPort = TriggerPort | (1<< TriggerBit)
#define CLR_TRIGGER TriggerPort = TriggerPort & ~(1<< TriggerBit)

//--------- starting of pulse length measuring ----------
void StartDistMeasuring(void)
{
	IC1count = 0;		// no capture event
	DistMeasReady = 0;

	Timer1Disable;
	TCNT1H = 0x00;		// clear pulse length timer
	TCNT1L = 0x00;
	InputCapture1IT_Clear;
	InputCapture1IT_Enable;
	Timer1Enable;      // enable and set capture sensing to rising edge
}


#define OCIE1A_IT_Enable TIMSK1 = TIMSK1 | (1 << OCIE1A)
#define OCIE1A_IT_Disable TIMSK1 = TIMSK1 & ~(1 << OCIE1A)
#define OC1IT_CLR TIFR1 = TIFR1 | (1 << OCF1A)

void StartTrigger(void)
{
	TriggerReady = 0;
	Timer1Disable;
	
	Timer1CaptureRisingEdge;
	InputCapture1IT_Clear;
	TCNT1 = 0x0000;			// clear timer1
	OCR1A = 20;				// set pulse length in uSec (timer1 clock 1MHz)
	SET_TRIGGER;			// setting to high of the trigger pulse
	OC1IT_CLR;
	OCIE1A_IT_Enable;		// Output compare IT after 10 uSec
	
	Timer1Enable;
}


// Clearing of trigger pulse
ISR(TIMER1_COMPA_vect)
{
	CLR_TRIGGER;			// end of the trigger pulse
	InputCapture1IT_Clear;
	OCIE1A_IT_Disable;
	TriggerReady = 1;
	//StartDistMeasuring();
}

//state codes
#define DistMeasStart 0
#define TriggWaiting 1
#define DistWaiting 2

//call parameters


#define FALSE 0
#define TRUE 1


DistMeasResult UH_DistanceMeas(unsigned char res)
{
	static unsigned char DistMeasState;
	unsigned int PulseLength;
	unsigned int N1;
	unsigned int N2;
	//static DistMeasResult DistResult;
	static volatile DistMeasResult DistResult;
	
	if(res)
	{
		InputCapture1IT_Disable;
		DistMeasState = DistMeasStart;
		DistResult.ready = FALSE;
	}
	else
	{
		switch(DistMeasState)
		{
			case DistMeasStart:
				DistResult.ready = FALSE;
				StartTrigger();
				DistMeasState = TriggWaiting;
				break;
			case TriggWaiting:
				if(TriggerReady)
				{
					StartDistMeasuring();
					StartTime1(MeasFaliure);			// because of echo failure
					DistMeasState = DistWaiting;
				}
				break;
			case DistWaiting:
				if(DistMeasReady)
				{								//compute frequency
					N1 = (NH1 << 8) + NL1;
					N2 = (NH2 << 8) + NL2;
					PulseLength = N2-N1;
				
					float DistanceFloat_cm = (float)PulseLength/uS_Pro_cm; //convert to cm
					DistanceInt_cm = (int) DistanceFloat_cm;
					DistResult.value = DistanceInt_cm;
					DistResult.valid = TRUE;
					DistResult.ready = TRUE;
					DistMeasState = DistMeasStart;
				}

				if(Time1Done())			// no echo pulse
				{
					DistMeasState = DistMeasStart;
					//DistanceInt_cm = 0;
					DistResult.valid = FALSE;
					DistResult.ready = TRUE;
					//StartTime2(MeasPeriod);
				}			
				break;
			default:
				DistMeasState = DistMeasStart;
			break;
		}
	}
	
	return(DistResult);
}


void UH_DistanceMeas_Init(void)
{
	TIMER0_Init();
	TIMER1_Init();
	CLR_TRIGGER;
	SetTriggerDir;
	UH_DistanceMeas(TRUE);
	InputCapture1IT_Clear;
}